"use strict";

/*
 * ATOM Appointment Management Plugin
 *
 * ©2018 Alexander Altendorfer / atomproductions
 * http://www.atomproductions.at
 */


(function () {

	//----------------//
	//   VARIABLES
	//----------------//

	var PLUGIN_PATH 	= aamlocalizevars.pluginpath,
		API_AJAXURL 	= aamlocalizevars.ajaxurl,
		LANGUAGE		= aamlocalizevars.language,
		MINTIME			= aamlocalizevars.min_time,
		MAXTIME			= aamlocalizevars.max_time,
		WEEKLIMIT		= aamlocalizevars.week_limit,
		DEBUG			= aamlocalizevars.debug,

		container,

		isAdmin,
		display,
		eventType,
		category,
		limit,

		calendar,
		loadingIndicator,
		loadingProblem,
		sendString,
		modal,
		confirmation,
		form,
		hdate,
		htime,
		dateFormat,
		firstLoad,
		timeout;

	//----------------//
	//    FUNCTIONS
	//----------------//

	//----------------//
	function calResizeHandler(view) {
	//----------------//

		if (calendar.width() < 480) {
			firstLoad = true;
			calendar.fullCalendar('changeView', 'agendaDay');
		} else {
			calendar.fullCalendar('changeView', 'agendaWeek');
		}

	}

	//----------------//
	function eventClickHandler(calEvent, jsEvent, view) {
	//----------------//

		if (calEvent.className == 'aam-full') {
			return false;
		}

		hdate = calEvent.start.format("LL");
		htime = calEvent.start.format("LT") + " - " + moment(calEvent.end).format("LT");

		modal.find('.atom-title').html(calEvent.title);
		modal.find('.atom-date').html(hdate);
		modal.find('.atom-time').html(htime);

		if (typeof(calEvent.moreinfo) !== 'undefined' && calEvent.moreinfo != "-1" && calEvent.moreinfo != ""  && calEvent.moreinfo != "#") {
			modal.find('.atom-moreinfo').show().attr('href', calEvent.moreinfo);
		} else {
			modal.find('.atom-moreinfo').hide();
		}

		jQuery('#atom-aam-form .atom-aam-input').blur(function() {
			validate(jQuery(this));
		});

		jQuery('#atom-aam-form-send').click(function() {
			formSendHandler(calEvent);
		});

		jQuery('.atom-aam-admin').hide();

		if (isAdmin) {
			if (calEvent.type == 'rule') {
				jQuery('#atom-admin-rule').show();

				var excbutton = modal.find('.atom-aam-single-exception');
				excbutton.click(function() {
					jQuery.ajax({
						type: "POST",
						url: API_AJAXURL,
						data: {
							action: 'atom_appointment_add_exception',
							begin: calEvent.start.format(),
							end: calEvent.end.format(),
							category: calEvent.category
						},
						success: function (data) {
							closeModal();
						}
					}).fail(function (jqXHR) {
						console.error("AJAX request failed");
					});
				});
				var excbuttonDay = modal.find('.atom-aam-day-exception');
				excbuttonDay.click(function() {
					jQuery.ajax({
						type: "POST",
						url: API_AJAXURL,
						data: {
							action: 'atom_appointment_add_exception',
							begin: calEvent.start.hour(0).minute(0).second(0).format(),
							end: calEvent.start.hour(0).minute(0).second(0).format()
						},
						success: function (data) {
							closeModal();
						}
					}).fail(function (jqXHR) {
						console.error("AJAX request failed");
					});
				});
				var excbuttonCategory = modal.find('.atom-aam-category-exception');
				if (calEvent.category != -1) {
					excbuttonCategory.show();
					excbuttonCategory.click(function() {
						jQuery.ajax({
							type: "POST",
							url: API_AJAXURL,
							data: {
								action: 'atom_appointment_add_exception',
								begin: calEvent.start.hour(0).minute(0).second(0).format(),
								end: calEvent.start.hour(0).minute(0).second(0).format(),
								category: calEvent.category
							},
							success: function (data) {
								closeModal();
							}
						}).fail(function (jqXHR) {
							console.error("AJAX request failed");
						});
					});
				} else {
					excbuttonCategory.hide();
				}
			} else if (calEvent.type == 'single') {
				jQuery('#atom-admin-single').show();

				var excbutton = modal.find('.atom-aam-remove-slot');
				excbutton.click(function() {
					if (confirm(jQuery(this).html() + '?')) {
						jQuery.ajax({
							type: "POST",
							url: API_AJAXURL,
							data: {
								action: 'atom_appointment_remove_slot',
								id: calEvent.db_id
							},
							success: function (data) {
								closeModal();
							}
						}).fail(function (jqXHR) {
							console.error("AJAX request failed");
						});
					}
				});

			} else if (calEvent.type == 'recurring') {
				jQuery('#atom-admin-recurring').show();

				var excbutton = modal.find('.atom-aam-remove-slot');
				excbutton.click(function() {
					if (confirm(jQuery(this).html() + '?')) {
						jQuery.ajax({
							type: "POST",
							url: API_AJAXURL,
							data: {
								action: 'atom_appointment_remove_slot',
								id: calEvent.db_id
							},
							success: function (data) {
								closeModal();
							}
						}).fail(function (jqXHR) {
							console.error("AJAX request failed");
						});
					}
				});
			}

		}

		modal.fadeIn();
		modal.addClass('open');
		if (display == 'calendar') container.css('min-height', jQuery('.atom-modal-content').height()+100);
	}

	function closeModal() {
		modal.removeClass('open');
		modal.fadeOut();
		confirmation.fadeOut();
		if (isAdmin) {
			modal.find('.atom-aam-category-exception').unbind('click');
			modal.find('.atom-aam-day-exception').unbind('click');
			modal.find('.atom-aam-single-exception').unbind('click');
			modal.find('.atom-aam-remove-slot').unbind('click');
		}
		jQuery('#atom-aam-form-send').unbind('click').html(sendString).prop('disabled', false);

		if (display == 'calendar') {
			calendar.fullCalendar('refetchEvents');
			container.css('min-height', 0);
		} else if (display == 'list') {
			rerenderListView();
		}
	}

	//----------------//
	function formSendHandler(event) {
	//----------------//

		var	inputFields = jQuery('#atom-aam-form .atom-aam-input'),
			values = [],
			valid = true;

		jQuery('#atom-aam-form-send').prop('disabled', true);

		inputFields.each(function(idx, el) {
			el = jQuery(el);
			if (validate(el)) {
				values.push({key:getFieldKey(el), value:el.val()});
			} else {
				valid = false;
			}

		});

		if (!validateDsgvo(jQuery('#atom-aam-consent'))) valid = false;

		if (valid) {
			sendFormData(values, event);
		} else {
			jQuery('#atom-aam-form-send').prop('disabled', false);
			jQuery('.atom-invalid-input').addClass('wiggle');
			setTimeout(function() {
				jQuery('.atom-invalid-input').removeClass('wiggle');
			}, 1000);
		}

	}

	//----------------//
	function validate(field) {
	//----------------//

		field = jQuery(field);

		var value = field.val(),
			type = field.attr('type'),
			valid = true,
			required = (field.attr('data-required') == 'true');

		switch (type) {
			case 'text':
			case 'textarea':
				valid = (!required || value.trim().length > 0)
				break;
			case 'email':
				var regex = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,})+$/;
				valid = (value.match(regex) || !required && value.trim().length == 0);
				break;
			case 'tel':
				var regex = /^[\s()+-]*([0-9][\s()+-]*){6,20}$/;
				valid = (value.match(regex) || !required && value.trim().length == 0);
				break;
			case 'select':
				valid = (value != null);
				break;
		}

		if (valid) {
			field.removeClass("atom-invalid-input");
		} else {
			field.val("");
			field.addClass("atom-invalid-input");
		}
		return valid;
	}

	//----------------//
	function validateDsgvo() {
	//----------------//

		var consent = jQuery('#atom-aam-consent');

		if (consent.length && !consent[0].checked) {
			consent.addClass("atom-invalid-input");
			return false;
		} else {
			consent.removeClass("atom-invalid-input");
			return true;
		}

	}

	//----------------//
	function getFieldKey(el) {
	//----------------//
		return el.attr('id').substr(15);
	}

	//----------------//
	function confirmBooking(values, event) {
	//----------------//

		// Google Analytics
		if (typeof ga !== 'undefined') {
			if (DEBUG) {console.log('Send ga event');}
			ga('send', 'event', 'appointment_booked', 'appointment_booked', event.title);
		}
		if (typeof gtag !== 'undefined') {
			if (DEBUG) {console.log('Send gtag event');}
			gtag('event', 'appointment_booked', {
				'event_category': 'appointment_booked',
				'event_label': event.title
			});
		}
		// FB Pixel
		if (typeof fbq !== 'undefined') {
			 fbq('track', 'Lead');
		}

		container.css('min-height', 0);
		jQuery('#atom-aam-modal-close').fadeOut();
		modal.find('.atom-email').html(jQuery('#atom-aam-input-field_email').val());
		confirmation.fadeIn();

	}

	//----------------//
	function goToFirstEvent() {
	//----------------//
		var start = moment();
		var end = moment().add('2', 'week');

		jQuery.ajax({
			type: "POST",
			url: API_AJAXURL,
			data: {
				action: 'atom_appointment_fetchevents',
				from: start.format('DD-MM-YYYY'),
				to: end.format('DD-MM-YYYY'),
				type: eventType,
				category: category
			},
			success: function (data) {
				data = JSON.parse(data);
				if (data.error) {
					console.error(data.error);
				} else if (typeof data.events[0] !== 'undefined') {
					calendar.fullCalendar('gotoDate', data.events[0].start);
				}
			}
		}).fail(function (jqXHR) {
			console.error("AJAX request failed");
		});
	}

	//----------------//
	function fetchEvents(start, end, timezone, callback) {
	//----------------//
		jQuery.ajax({
			type: "POST",
			url: API_AJAXURL,
			data: {
				action: 'atom_appointment_fetchevents',
				from: start.format('YYYY-MM-DD'),
				to: end.format('YYYY-MM-DD'),
				type: eventType,
				category: category
			},
			success: function (data) {
				data = JSON.parse(data);
				if (DEBUG) console.log(data);
				if (data.error) {
					console.error(data.error);
					setTimeout(function () {
						fetchEvents(start, end, timezone, callback);
					}, 1000);
				} else {
					callback(data.events);
				}
			}
		}).fail(function (jqXHR) {
			console.error("AJAX request failed");
		});

		if (moment().isBetween(start, end)) {
			jQuery('.fc-prev-button').attr('disabled', 'disabled');
		} else {
			jQuery('.fc-prev-button').removeAttr('disabled');
		}

		if (end.isAfter(moment().add('6', 'week'))) {
			jQuery('.fc-next-button').attr('disabled', 'disabled');
		} else {
			jQuery('.fc-next-button').removeAttr('disabled');
		}
	}
	//----------------//
	function indicateLoading(isLoading, view) {
	//----------------//

		if (isLoading) {
			loadingIndicator.show();

			timeout = setTimeout(function() {
				loadingProblem.fadeIn();
			}, 10000);

		} else {
			loadingIndicator.fadeOut();
			clearTimeout(timeout);
			loadingProblem.fadeOut();
		}

	}

	//----------------//
	function sendFormData(values, event) {
	//----------------//

		var errorString = jQuery('#aam-button-error-string').html(),
			loadingString = jQuery('#aam-button-loading-string').html();

		var time1 = setTimeout(function(){
			jQuery('#atom-aam-form-send').html(loadingString);
		}, 500);
		var time2 = setTimeout(function(){
			jQuery('#atom-aam-form-send').prop('disabled', false).html(errorString);
		}, 20000);

		if (event.db_id) {
			var slot_id = event.db_id;
		} else {
			var slot_id = -1;
		}

		jQuery.ajax({
			type: "POST",
			url: API_AJAXURL,
			data: {
				action: 'atom_appointment_formsubmit',
				startdate: event.start.format(),
				enddate: event.end.format(),
				values: values,
				slot_id: slot_id,
				title: event.title,
				category: event.category
			},
			success: function(data) {
				data = JSON.parse(data);
				clearTimeout(time1);
				clearTimeout(time2);
				if (data.success) {
					confirmBooking(values, event);
				} else {
					console.error(data.error);
					jQuery('#atom-aam-form-send').prop('disabled', false).html(errorString);
				}
			}
		}).fail(function (jqXHR) {
			jQuery('#atom-aam-form-send').prop('disabled', false).html(errorString);
			console.error("AJAX request failed");
		});
	}

	//----------------//
	function validRange(currentDate) {
	//----------------//
        return {
            start: currentDate.clone(),
            end: currentDate.clone().add(WEEKLIMIT, 'weeks').endOf('week')
        };
	}

	//----------------//
	function rerenderListView() {
	//----------------//

		loadingIndicator.fadeIn();
		jQuery.ajax({
			type: "POST",
			url: API_AJAXURL,
			data: {
				action: 'atom_appointment_get_frontend_view',
				view: 'list-inner',
				from: moment().format('YYYY-MM-DD'),
				to: moment().add(2, 'week').format('YYYY-MM-DD'),
				type: eventType,
				category: category,
				limit: limit
			},
			success: function(data) {
				container.find('#atom-aam-list').html(data);
				loadingIndicator.fadeOut();
				container.find('.atom-aam-event').click(function() {
					eventClickHandler({
						start:	moment(jQuery(this).attr('data-start')),
						end:	moment(jQuery(this).attr('data-end')),
						title:	jQuery(this).attr('data-title'),
						type:	jQuery(this).attr('data-type'),
						db_id:	jQuery(this).attr('data-db_id'),
					});
				});
			}
		}).fail(function (jqXHR) {
			console.error("AJAX request failed");
		});

	}

	//----------------//
	function initCalendarView() {
	//----------------//

		jQuery.ajax({
			type: "POST",
			url: API_AJAXURL,
			data: {
				action: 'atom_appointment_get_frontend_view',
				view: 'calendar'
			},
			success: function(data) {
				container.html(data);

				calendar = jQuery('#atom-aam-calendar');
				loadingIndicator = jQuery('#atom-aam-loadingindicator');
				loadingProblem = jQuery('#atom-aam-loading-problem');
				sendString = jQuery('#atom-aam-form-send').html();
				modal = jQuery('#atom-aam-modal');
				confirmation = modal.find('#atom-aam-confirmation');

				jQuery('#atom-aam-modal-close').click(function() {
					closeModal();
				});

				jQuery(document).keydown(function(event) {
					if (event.keyCode == 27 && modal.css('display') != 'none') {
						closeModal();
					}
				});

				firstLoad = true;

				calendar.fullCalendar({
					defaultView: 		'agendaWeek',
					locale: 			LANGUAGE,
					allDaySlot: 		false,
					contentHeight: 		'auto',
					height: 			'parent',
					columnFormat: 		dateFormat,
					slotLabelFormat: 	'LT',
					slotDuration: 		'01:00:00',
					minTime:			MINTIME,
					maxTime: 			MAXTIME,
					firstDay: 			1,
					eventClick: 		eventClickHandler,
					windowResize: 		calResizeHandler,
					events: 			fetchEvents,
					loading: 			indicateLoading,
					validRange:			validRange
				});
				calResizeHandler();
				goToFirstEvent();
			}
		});

	}

	//----------------//
	function initListView() {
	//----------------//

		jQuery.ajax({
			type: "POST",
			url: API_AJAXURL,
			data: {
				action: 'atom_appointment_get_frontend_view',
				view: 'list',
				from: moment().format('YYYY-MM-DD'),
				to: moment().add(2, 'week').format('YYYY-MM-DD'),
				type: eventType,
				category: category,
				limit: limit
			},
			success: function(data) {
				container.append(data);
				loadingIndicator = jQuery('.atom-aam-loadingindicator').fadeOut();
				sendString = jQuery('#atom-aam-form-send').html();
				modal = jQuery('#atom-aam-modal');
				confirmation = modal.find('#atom-aam-confirmation');
				container.find('.atom-aam-event').click(function() {
					eventClickHandler({
						start:	moment(jQuery(this).attr('data-start')),
						end:	moment(jQuery(this).attr('data-end')),
						title:	jQuery(this).attr('data-title'),
						type:	jQuery(this).attr('data-type'),
						db_id:	jQuery(this).attr('data-db_id'),
					});
				});

				jQuery('#atom-aam-modal-close').click(function() {
					closeModal();
				});

				jQuery(document).keydown(function(event) {
					if (event.keyCode == 27 && modal.css('display') != 'none') {
						closeModal();
					}
				});

			}
		}).fail(function (jqXHR) {
			console.error("AJAX request failed");
		});

	}

	//----------------//
	(function init() {
	//----------------//

		jQuery(document).ready(function () {

			if (LANGUAGE == 'de') {
				dateFormat = 'dd\n DD.MM.';
			} else {
				dateFormat = 'dd\n MM/DD';
			}

			container = jQuery('#atom-appointment-management');
			isAdmin = container.attr('data-admin') == "true";
			display = (container.attr('data-display')) ? container.attr('data-display') : 'calendar';
			eventType = (container.attr('data-type')) ? container.attr('data-type') : 'all';
			category = (container.attr('data-category')) ? container.attr('data-category') : 'all';
			limit = container.attr('data-limit');

			if (DEBUG) console.warn('ATOM APPOINTMENT MANAGEMENT: Debug mode enabled');

			switch (display) {
				case 'list':
					initListView();
					break;
				case 'calendar':
				default:
					initCalendarView();
					break;
			}

		});

	})();

})();
